@@ -18,7 +18,7 @@ form representation type.
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
@@ -40,7 +40,7 @@ from ufl.algorithms import extract_elements, extract_sub_elements
 from ufl.algorithms import extract_common_cell
 
 # FFC modules
-from ffc.log import log, info, begin, end, warning, debug, error, ffc_assert
+from ffc.log import log, info, begin, end, warning, debug, error, ffc_assert, warning_blue
 from ffc.utils import all_equal
 from ffc.quadratureelement import default_quadrature_degree
 from ffc.utils import all_equal
@@ -176,6 +176,10 @@ def _attach_integral_metadata(form_data, common_cell, parameters):
                 if not key in integral_metadata:
                     integral_metadata[key] = parameters[key]
 
+            # Special case: handling -1 as "auto" for quadrature_degree
+            if integral_metadata["quadrature_degree"] == -1:
+                integral_metadata["quadrature_degree"] = "auto"
+
             # Check metadata
             r  = integral_metadata["representation"]
             qd = integral_metadata["quadrature_degree"]
@@ -188,6 +192,7 @@ def _attach_integral_metadata(form_data, common_cell, parameters):
                 if not qd >= 0:
                     info("Valid choices are nonnegative integers or 'auto'.")
                     error("Illegal quadrature degree for integral: " + str(qd))
+                integral_metadata["quadrature_degree"] = qd
             if not qr in ("default", "canonical", "auto"):
                 info("Valid choices are 'default', 'canonical' or 'auto'.")
                 error("Illegal choice of quadrature rule for integral: " + str(qr))
@@ -208,8 +213,10 @@ def _attach_integral_metadata(form_data, common_cell, parameters):
                                                     form_data.unique_sub_elements)
                 info("quadrature_degree: auto --> %d" % qd)
                 integral_metadata["quadrature_degree"] = qd
+                _check_quadrature_degree(qd, form_data.topological_dimension)
             else:
                 info("quadrature_degree: %d" % qd)
+                _check_quadrature_degree(qd, form_data.topological_dimension)
 
             # Automatic selection of quadrature rule
             if qr == "auto":
@@ -332,7 +339,7 @@ def _auto_select_degree(elements):
     """
 
     # Extract common degree
-    common_degree = max([e.degree() for e in elements])
+    common_degree = max([e.degree() for e in elements] or [None])
     if common_degree is None:
         common_degree = default_quadrature_degree
 
@@ -397,3 +404,12 @@ def _auto_select_quadrature_degree(integral, representation, elements):
     debug("Selecting quadrature degree based on total polynomial degree of integrand: " + str(q))
 
     return q
+
+def _check_quadrature_degree(degree, top_dim):
+    """Check that quadrature degree does not result in a unreasonable high
+    number of integration points."""
+    num_points = ((degree + 1 + 1) / 2)**top_dim
+    if num_points >= 100:
+        warning_blue("WARNING: The number of integration points for each cell will be: %d" % num_points)
+        warning_blue("         Consider using the option 'quadrature_degree' to reduce the number of points")
+
