@@ -17,16 +17,16 @@ UFC function from an (optimized) intermediate representation (OIR).
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Mehdi Nikbakht, 2010
 #
 # First added:  2009-12-16
-# Last changed: 2011-02-21
+# Last changed: 2011-11-28
 
 # FFC modules
 from ffc.log import info, begin, end, debug_code
@@ -64,11 +64,11 @@ def generate_code(ir, prefix, parameters):
     ir_elements, ir_dofmaps, ir_integrals, ir_forms = ir
 
     # Generate code for elements
-    info("Generating code for %d elements" % len(ir_elements))
+    info("Generating code for %d element(s)" % len(ir_elements))
     code_elements = [_generate_element_code(ir, prefix, parameters) for ir in ir_elements]
 
     # Generate code for dofmaps
-    info("Generating code for %d dofmaps" % len(ir_dofmaps))
+    info("Generating code for %d dofmap(s)" % len(ir_dofmaps))
     code_dofmaps = [_generate_dofmap_code(ir, prefix, parameters) for ir in ir_dofmaps]
 
     # Generate code for integrals
@@ -271,8 +271,9 @@ def _init_mesh(ir):
     num_dofs = ir[0]
     component = format["component"]
     entities =  format["num entities"]
-    dimension = format["inner product"](num_dofs, [component(entities, d)
-                                                   for d in range(len(num_dofs))])
+    dimension = format["inner product"]([format["int"](d) for d in num_dofs],
+                                        [component(entities, d)
+                                         for d in range(len(num_dofs))])
     # Handle global "elements" if any
     if ir[1]:
         dimension = format["add"]([dimension, format["int"](ir[1])])
@@ -281,8 +282,9 @@ def _init_mesh(ir):
         except:
             pass
 
-    return "\n".join([format["assign"](format["member global dimension"], dimension),
+    code = "\n".join([format["assign"](format["member global dimension"], dimension),
                       format["return"](format["bool"](False))])
+    return code
 
 def _tabulate_facet_dofs(ir):
     "Generate code for tabulate_facet_dofs."
@@ -329,7 +331,8 @@ def _tabulate_dofs(ir):
         # Handle fakes (Space of reals)
         if fakes[no] and num_dofs_per_element[no] == 1:
             code.append(assign(component(dofs_variable, i), offset_name))
-            code.append(iadd(offset_name, 1))
+            if offset_name != "0":
+                code.append(iadd(offset_name, 1))
             i += 1
             continue
 
@@ -386,7 +389,7 @@ def _tabulate_coordinates(ir):
         w = coefficients(coordinate)
         for j in range(cell_dim):
             # Compute physical coordinate
-            coords = [component(f_x, (k, j)) for k in range(cell_dim + 1)]
+            coords = [component(f_x(), (k, j)) for k in range(cell_dim + 1)]
             value = inner_product(w, coords)
 
             # Assign coordinate
@@ -465,7 +468,7 @@ def _postprocess_code(code, parameters):
 def _indent_code(code):
     "Indent code that should be indented."
     for key in code:
-        if not key in ("classname", "members", "constructor_arguments", "initializer_list"):
+        if not key in ("classname", "members", "constructor_arguments", "initializer_list", "additional_includes_set"):
             code[key] = indent(code[key], 4)
 
 def _remove_code(code, parameters):
