@@ -1,6 +1,6 @@
 "This module defines rules and algorithms for generating C++ code."
 
-# Copyright (C) 2009-2010 Anders Logg
+# Copyright (C) 2009-2011 Anders Logg
 #
 # This file is part of FFC.
 #
@@ -11,17 +11,17 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Kristian B. Oelgaard 2010
+# Modified by Kristian B. Oelgaard 2011
 # Modified by Marie E. Rognes 2010
 #
 # First added:  2009-12-16
-# Last changed: 2011-02-21
+# Last changed: 2011-11-28
 
 # Python modules
 import re, numpy, platform
@@ -50,11 +50,14 @@ format.update({
     "loop":           lambda i, j, k: "for (unsigned int %s = %s; %s < %s; %s++)"% (i, j, i, k, i),
     "generate loop":  lambda v, w, _indent=0: _generate_loop(v, w, _indent),
     "is equal":       " == ",
-    "not equal":      " == ",
+    "not equal":      " != ",
     "less than":      " < ",
     "greater than":   " > ",
     "less equal":     " <= ",
     "greater equal":  " >= ",
+    "and":            " && ",
+    "or":             " || ",
+    "not":            lambda v: "!(%s)" % v,
     "do nothing":     "// Do nothing"
 })
 
@@ -92,6 +95,11 @@ format.update({
     "acos":           lambda v: "std::acos(%s)" % str(v),
     "asin":           lambda v: "std::asin(%s)" % str(v),
     "atan":           lambda v: "std::atan(%s)" % str(v),
+    "erf":            lambda v:  "erf(%s)" % str(v),
+    "bessel_i":       lambda v, n:  "boost::math::tr1::cyl_bessel_i(%s, %s)" % (str(n), str(v)),
+    "bessel_j":       lambda v, n:  "boost::math::tr1::cyl_bessel_j(%s, %s)" % (str(n), str(v)),
+    "bessel_k":       lambda v, n:  "boost::math::tr1::cyl_bessel_k(%s, %s)" % (str(n), str(v)),
+    "bessel_y":       lambda v, n:  "boost::math::tr1::cyl_neumann(%s, %s)" % (str(n), str(v)),
     "absolute value": lambda v: "std::abs(%s)" % str(v),
     "sqrt":           lambda v: "std::sqrt(%s)" % str(v),
     "addition":       lambda v: _add(v),
@@ -117,6 +125,7 @@ format.update({
     "det(J)":           lambda r=None: "detJ%s" % choose_map[r],
     "cell volume":      lambda r=None: "volume%s" % choose_map[r],
     "circumradius":     lambda r=None: "circumradius%s" % choose_map[r],
+    "facet area":       "facet_area",
     "scale factor":     "det",
     "transform":        lambda t, j, k, r: _transform(t, j, k, r),
     "normal component": lambda r, j: "n%s%s" % (choose_map[r], j),
@@ -125,7 +134,7 @@ format.update({
     "z coordinate":     "Z",
     "ip coordinates":   lambda i, j: "X%d[%d]" % (i, j),
     "affine map table": lambda i, j: "FEA%d_f%d" % (i, j),
-    "coordinates":      "x"
+    "coordinates":      lambda r=None: "x%s" % choose_map[r]
 })
 
 # UFC function arguments and class members (names)
@@ -230,6 +239,7 @@ format.update({
     "generate normal":      lambda d, i: _generate_normal(d, i),
     "generate cell volume": lambda d, i: _generate_cell_volume(d, i),
     "generate circumradius": lambda d, i: _generate_circumradius(d, i),
+    "generate facet area":  lambda d: facet_area[d],
     "generate ip coordinates":  lambda g, num_ip, name, ip, r=None: (ip_coordinates[g][0], ip_coordinates[g][1] % \
                                 {"restriction": choose_map[r], "ip": ip, "name": name, "num_ip": num_ip}),
     "scale factor snippet": scale_factor,
@@ -309,6 +319,10 @@ def _multiply(factors):
         if f == "0":
             return cpp_str(0)
 
+        # If f is 1, don't add it to list of factors
+        if f == "1":
+            continue
+
         # If sum-like, parentheseze factor
         if "+" in f or "-" in f:
             f = "(%s)" % f
@@ -614,19 +628,26 @@ def set_float_formatting(precision):
     "Set floating point formatting based on precision."
 
     # Options for float formatting
-    f1 = "%%.%df" % precision
-    f2 = "%%.%de" % precision
+    #f1     = "%%.%df" % precision
+    #f2     = "%%.%de" % precision
+    f1     = "%%.%dg" % precision
+    f2     = "%%.%dg" % precision
+    f_int  = "%%.%df" % 1
+
+    eps = eval("1e-%s" % precision)
 
     # Regular float formatting
     def floating_point_regular(v):
-        if abs(v) < 100.0:
+        if abs(v - round(v, 1)) < eps:
+            return f_int % v
+        elif abs(v) < 100.0:
             return f1 % v
         else:
             return f2 % v
 
     # Special float formatting on Windows (remove extra leading zero)
     def floating_point_windows(v):
-        return floating_point(v).replace("e-0", "e-").replace("e+0", "e+")
+        return floating_point_regular(v).replace("e-0", "e-").replace("e+0", "e+")
 
     # Set float formatting
     if platform.system() == "Windows":
@@ -658,7 +679,7 @@ types = [["double"],
          ["const", "unsigned", "int"]]
 
 # Special characters and delimiters
-special_characters = ["+", "-", "*", "/", "=", ".", " ", ";", "(", ")", "\\", "{", "}", "[","]"]
+special_characters = ["+", "-", "*", "/", "=", ".", " ", ";", "(", ")", "\\", "{", "}", "[","]", "!"]
 
 def remove_unused(code, used_set=set()):
     """
