@@ -12,14 +12,22 @@ code which is more or less a C++ representation of the code found in FIAT."""
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2007-04-04
 # Last changed: 2011-02-21
+#
+# Modified by Marie E. Rognes, 2011
+#
+# MER: The original module generated code that was more or less a C++
+# representation of the code found in FIAT. I've modified this (for 2
+# and 3D) to generate code that does the same as FIAT, but with loops
+# unrolled etc, thus removing unnecessary computations at runtime.
+# There might be some clean-ups required, specially after this.
 
 # Python modules
 import math
@@ -400,72 +408,78 @@ def _compute_values(data, dof_data):
 
     return code
 
-# FIAT_NEW code (compute index function) TriangleExpansionSet.
-# def idx(p,q):
-#    return (p+q)*(p+q+1)/2 + q
-def _idx2D(p, q):
-    f_add   = format["add"]
-    f_group = format["grouping"]
-    f_int   = format["int"]
-    pq = format["addition"]([str(p), str(q)])
-    pq1 = f_group(f_add([pq, f_int(1)]))
-    if q == "0":
-        return format["div"](format["mul"]([pq, pq1]), f_int(2))
-    return f_add([format["div"](format["mul"]([f_group(pq), pq1]), f_int(2)), str(q)])
+# MER: Uncommented these after unrolling loops at compile time instead
+# of runtime
+#
+# # FIAT_NEW code (compute index function) TriangleExpansionSet.
+# # def idx(p,q):
+# #    return (p+q)*(p+q+1)/2 + q
+# def _idx2D(p, q):
+#     f_add   = format["add"]
+#     f_group = format["grouping"]
+#     f_int   = format["int"]
+#     pq = format["addition"]([str(p), str(q)])
+#     pq1 = f_group(f_add([pq, f_int(1)]))
+#     if q == "0":
+#         return format["div"](format["mul"]([pq, pq1]), f_int(2))
+#     return f_add([format["div"](format["mul"]([f_group(pq), pq1]), f_int(2)), str(q)])
 
-# FIAT_NEW code (compute index function) TetrahedronExpansionSet.
-# def idx(p,q,r):
-#     return (p+q+r)*(p+q+r+1)*(p+q+r+2)/6 + (q+r)*(q+r+1)/2 + r
-def _idx3D(p, q, r):
-    f_add   = format["add"]
-    f_group = format["grouping"]
-    f_int   = format["int"]
-    pqr  = format["addition"]([str(p), str(q), str(r)])
-    pqr1 = f_group(f_add([pqr, f_int(1)]))
-    pqr2 = f_group(f_add([pqr, f_int(2)]))
-    qr   = format["addition"]([str(q), str(r)])
-    qr1  = f_group(f_add([qr, f_int(1)]))
-    if q == r == "0":
-        return format["div"](format["mul"]([pqr, pqr1, pqr2]), f_int(6))
-    pqrg = f_group(pqr)
-    fac0 = format["div"](format["mul"]([pqrg, pqr1, pqr2]), f_int(6))
-    if r == "0":
-        return f_add([fac0, format["div"](format["mul"]([qr, qr1]), f_int(2))])
-    return f_add([fac0, format["div"](format["mul"]([f_group(qr), qr1]), f_int(2)), str(r)])
+# # FIAT_NEW code (compute index function) TetrahedronExpansionSet.
+# # def idx(p,q,r):
+# #     return (p+q+r)*(p+q+r+1)*(p+q+r+2)/6 + (q+r)*(q+r+1)/2 + r
+# def _idx3D(p, q, r):
+#     f_add   = format["add"]
+#     f_group = format["grouping"]
+#     f_int   = format["int"]
+#     pqr  = format["addition"]([str(p), str(q), str(r)])
+#     pqr1 = f_group(f_add([pqr, f_int(1)]))
+#     pqr2 = f_group(f_add([pqr, f_int(2)]))
+#     qr   = format["addition"]([str(q), str(r)])
+#     qr1  = f_group(f_add([qr, f_int(1)]))
+#     if q == r == "0":
+#         return format["div"](format["mul"]([pqr, pqr1, pqr2]), f_int(6))
+#     pqrg = f_group(pqr)
+#     fac0 = format["div"](format["mul"]([pqrg, pqr1, pqr2]), f_int(6))
+#     if r == "0":
+#         return f_add([fac0, format["div"](format["mul"]([qr, qr1]), f_int(2))])
+#     return f_add([fac0, format["div"](format["mul"]([f_group(qr), qr1]), f_int(2)), str(r)])
 
-# FIAT_NEW code (helper variables) TriangleExpansionSet and TetrahedronExpansionSet.
-# def jrc( a , b , n ):
-#    an = float( ( 2*n+1+a+b)*(2*n+2+a+b)) \
-#        / float( 2*(n+1)*(n+1+a+b))
+# # FIAT_NEW code (helper variables) TriangleExpansionSet and TetrahedronExpansionSet.
+# # def jrc( a , b , n ):
+# #    an = float( ( 2*n+1+a+b)*(2*n+2+a+b)) \
+# #        / float( 2*(n+1)*(n+1+a+b))
 
-#    bn = float( (a*a-b*b) * (2*n+1+a+b) ) \
-#        / float( 2*(n+1)*(2*n+a+b)*(n+1+a+b) )
+# #    bn = float( (a*a-b*b) * (2*n+1+a+b) ) \
+# #        / float( 2*(n+1)*(2*n+a+b)*(n+1+a+b) )
 
-#    cn = float( (n+a)*(n+b)*(2*n+2+a+b)  ) \
-#        / float( (n+1)*(n+1+a+b)*(2*n+a+b) )
-#    return an,bn,cn
-def _jrc(a, b, n):
-    f1        = create_float(1)
-    f2        = create_float(2)
-    an_num    = create_product([ f2*n + f1 + a + b, f2*n + f2 + a + b ])
-    an_denom  = create_product([ f2, n + f1, n + f1 + a + b ])
-    an        = create_fraction(an_num, an_denom)
+# #    cn = float( (n+a)*(n+b)*(2*n+2+a+b)  ) \
+# #        / float( (n+1)*(n+1+a+b)*(2*n+a+b) )
+# #    return an,bn,cn
+# def _jrc(a, b, n):
+#     f1        = create_float(1)
+#     f2        = create_float(2)
+#     an_num    = create_product([ f2*n + f1 + a + b, f2*n + f2 + a + b ])
+#     an_denom  = create_product([ f2, n + f1, n + f1 + a + b ])
+#     an        = create_fraction(an_num, an_denom)
 
-    bn_num    = create_product([ a*a - b*b, f2*n + f1 + a + b ])
-    bn_denom  = create_product([ f2, n + f1, f2*n + a + b, n + f1 + a + b ])
-    bn        = create_fraction(bn_num, bn_denom)
+#     bn_num    = create_product([ a*a - b*b, f2*n + f1 + a + b ])
+#     bn_denom  = create_product([ f2, n + f1, f2*n + a + b, n + f1 + a + b ])
+#     bn        = create_fraction(bn_num, bn_denom)
 
-    cn_num    = create_product([ n + a, n + b, f2*n + f2 + a + b ])
-    cn_denom  = create_product([ n + f1, n + f1 + a + b, f2*n + a + b ])
-    cn        = create_fraction(cn_num, cn_denom)
-    return (an, bn, cn)
+#     cn_num    = create_product([ n + a, n + b, f2*n + f2 + a + b ])
+#     cn_denom  = create_product([ n + f1, n + f1 + a + b, f2*n + a + b ])
+#     cn        = create_fraction(cn_num, cn_denom)
+#     return (an, bn, cn)
 
 def _compute_basisvalues(data, dof_data):
     """From FIAT_NEW.expansions."""
 
+    UNROLL = True
+
     # Prefetch formats to speed up code generation.
     f_comment     = format["comment"]
     f_add         = format["add"]
+    f_mul         = format["mul"]
     f_imul        = format["imul"]
     f_sub         = format["sub"]
     f_group       = format["grouping"]
@@ -544,6 +558,12 @@ def _compute_basisvalues(data, dof_data):
     # FIXME: KBO: Change this when supporting R^2 in R^3 elements.
     element_cell_domain = data["cell_domain"]
 
+    def _jrc(a, b, n):
+        an = float( ( 2*n+1+a+b)*(2*n+2+a+b))/ float( 2*(n+1)*(n+1+a+b))
+        bn = float( (a*a-b*b) * (2*n+1+a+b))/ float( 2*(n+1)*(2*n+a+b)*(n+1+a+b) )
+        cn = float( (n+a)*(n+b)*(2*n+2+a+b))/ float( (n+1)*(n+1+a+b)*(2*n+a+b) )
+        return (an,bn,cn)
+
     # 1D
     if (element_cell_domain == "interval"):
         # FIAT_NEW.expansions.LineExpansionSet.
@@ -582,29 +602,19 @@ def _compute_basisvalues(data, dof_data):
                 #    a4 = a4 / a1
                 #    result[k,:] = ( a2 + a3 * xsnew ) * result[k-1,:] \
                 #        - a4 * result[k-2,:]
-                # Declare helper variables (Note the a2 is always zero and therefore left out).
-                code += [f_decl(f_double, str(f1), f_float(0))]
-                code += [f_decl(f_double, str(f2), f_float(0))]
-                code += [f_decl(f_double, str(f3), f_float(0))]
-                lines = []
-                loop_vars = [(str(symbol_p), 2, int_n1)]
-                # Create names.
-                basis_k   = create_symbol(f_component(f_basisvalue, str(symbol_p)), CONST)
-                basis_km1 = create_symbol(f_component(f_basisvalue, f_sub([str(symbol_p), int_1])), CONST)
-                basis_km2 = create_symbol(f_component(f_basisvalue, f_sub([str(symbol_p), int_2])), CONST)
-                # Compute helper variables.
-                a1 = create_product([float_2, symbol_p, symbol_p, float_2*symbol_p - float_2])
-                a3 = create_fraction(create_product([float_2*symbol_p,\
-                                                     float_2*symbol_p - float_2,
-                                                     float_2*symbol_p - float_1]), a1)
-                a4 = create_fraction(create_product([float_4*symbol_p, symbol_p - float_1, symbol_p - float_1]), a1)
-                lines.append(f_assign(str(f1), a1 ))
-                lines.append(f_assign(str(f2), a3 ))
-                lines.append(f_assign(str(f3), a4 ))
-                # Compute value.
-                lines.append(f_assign(str(basis_k), create_product([f2, symbol_x*basis_km1]) - f3*basis_km2))
-                # Create loop (block of lines).
-                code += f_loop(lines, loop_vars)
+
+                # The below implements the above (with a = b = apb = 0)
+                for r in range(2, embedded_degree+1):
+
+                    # Define helper variables
+                    a1 = 2.0*r*r*(2.0*r - 2.0)
+                    a3 = ((2.0*r - 2.0)*(2.0*r - 1.0 )*(2.0*r))/a1
+                    a4 = (2.0*(r - 1.0)*(r - 1.0)*(2.0*r))/a1
+
+                    assign_to = f_component(f_basisvalue, r)
+                    assign_from = f_sub([f_mul([f_x, f_component(f_basisvalue, r-1), f_float(a3)]),
+                                         f_mul([f_component(f_basisvalue, r-2), f_float(a4)])])
+                    code += [f_assign(assign_to, assign_from)]
 
         # Scale values.
         # FIAT_NEW.expansions.LineExpansionSet.
@@ -643,6 +653,9 @@ def _compute_basisvalues(data, dof_data):
         #    results[0,ii] = 1.0 + apts[ii,0]-apts[ii,0]+apts[ii,1]-apts[ii,1]
         code += [f_assign(f_component(f_basisvalue, 0), f_float(1.0))]
 
+        def _idx2d(p, q):
+            return (p+q)*(p+q+1)/2 + q
+
         # Only continue if the embedded degree is larger than zero.
         if embedded_degree > 0:
 
@@ -661,36 +674,37 @@ def _compute_basisvalues(data, dof_data):
                 #    results[idx(p+1,0)] = a * f1 * results[idx(p,0),:] \
                 #        - p/(1.0+p) * f3 *results[idx(p-1,0),:]
                 # FIXME: KBO: Is there an error in FIAT? why is b not used?
-                lines = []
-                loop_vars = [(str(symbol_p), 1, embedded_degree)]
-                # Compute indices
-                lines.append(f_assign(idx0, _idx2D(f_group(f_add([str(symbol_p), int_1])), int_0)))
-                lines.append(f_assign(idx1, _idx2D(symbol_p, int_0)))
-                lines.append(f_assign(idx2, _idx2D(f_group(f_sub([str(symbol_p), int_1])), int_0)))
-                # Compute single helper variable an.
-                lines.append(f_assign(str(an), create_fraction(float_2*symbol_p + float_1, symbol_p + float_1)))
-                # Compute value.
-                fac0 = create_product([an, f1, basis_idx1])
-                fac1 = create_product([create_fraction(symbol_p, float_1 + symbol_p), f3, basis_idx2])
-                lines.append(f_assign(str(basis_idx0), fac0 - fac1))
-                # Create loop (block of lines).
-                code += f_loop(lines, loop_vars)
+
+                for r in range(1, embedded_degree):
+                    rr = _idx2d((r + 1), 0)
+                    assign_to = f_component(f_basisvalue, rr)
+                    ss = _idx2d(r, 0)
+                    tt = _idx2d((r - 1), 0)
+                    A = (2*r + 1.0)/(r + 1)
+                    B = r/(1.0 + r)
+                    v1 = f_mul([f_component(f_basisvalue, ss), f_float(A),
+                                str(f1)])
+                    v2 = f_mul([f_component(f_basisvalue, tt), f_float(B),
+                                str(f3)])
+                    assign_from = f_sub([v1, v2])
+                    code += [f_assign(assign_to, assign_from)]
 
             # FIAT_NEW code (loop 2 in FIAT).
             # for p in range(n):
             #    results[idx(p,1),:] = 0.5 * (1+2.0*p+(3.0+2.0*p)*y) \
             #        * results[idx(p,0)]
-            lines = []
-            loop_vars = [(str(symbol_p), 0, embedded_degree)]
-            # Compute indices.
-            lines.append(f_assign(idx0, _idx2D(symbol_p, int_1)))
-            lines.append(f_assign(idx1, _idx2D(symbol_p, int_0)))
-            # Compute value.
-            fac0 = create_product([float_3 + float_2*symbol_p, symbol_y])
-            fac1 = create_product([float_0_5, float_1 + float_2*symbol_p + fac0, basis_idx1])
-            lines.append(f_assign(str(basis_idx0), fac1.expand().reduce_ops()))
-            # Create loop (block of lines).
-            code += f_loop(lines, loop_vars)
+
+            for r in range(0, embedded_degree):
+                # (p+q)*(p+q+1)/2 + q
+                rr = _idx2d(r, 1)
+                assign_to = f_component(f_basisvalue, rr)
+                ss = _idx2d(r, 0)
+                A = 0.5*(1 + 2*r)
+                B = 0.5*(3 + 2*r)
+                C = f_add([f_float(A), f_mul([f_float(B), str(symbol_y)])])
+                assign_from = f_mul([f_component(f_basisvalue, ss),
+                                     f_group(C)])
+                code += [f_assign(assign_to, assign_from)]
 
             # Only active is embedded_degree > 1.
             if embedded_degree > 1:
@@ -701,43 +715,39 @@ def _compute_basisvalues(data, dof_data):
                 #        results[idx(p,q+1),:] \
                 #            = ( a1 * y + a2 ) * results[idx(p,q)] \
                 #            - a3 * results[idx(p,q-1)]
-                lines = []
-                loop_vars = [(str(symbol_p), 0, int_nm1),\
-                             (str(symbol_q), 1, f_sub([int_n, str(symbol_p)]))]
-                # Compute indices.
-                lines.append(f_assign(idx0, _idx2D(symbol_p, f_add([str(symbol_q), int_1]) )))
-                lines.append(f_assign(idx1, _idx2D(symbol_p, symbol_q)))
-                lines.append(f_assign(idx2, _idx2D(symbol_p, f_sub([str(symbol_q), int_1]))))
-                # Comute all helper variables.
-                jrc = _jrc(float_2*symbol_p + float_1, float_0, symbol_q)
-                lines.append(f_assign(str(an), jrc[0]))
-                lines.append(f_assign(str(bn), jrc[1]))
-                lines.append(f_assign(str(cn), jrc[2]))
-                # Compute value.
-                fac0 = create_product([an * symbol_y + bn, basis_idx1])
-                fac1 = cn * basis_idx2
-                lines.append(f_assign(str(basis_idx0), fac0 - fac1))
-                # Create loop (block of lines).
-                code += f_loop(lines, loop_vars)
+
+                for r in range(0, embedded_degree - 1):
+                    for s in range(1, embedded_degree - r):
+                        rr = _idx2d(r, (s + 1))
+                        ss = _idx2d(r, s)
+                        tt = _idx2d(r, s - 1)
+                        A, B, C = _jrc(2*r + 1, 0, s)
+                        assign_to = f_component(f_basisvalue, rr)
+                        assign_from = f_sub([f_mul([f_component(f_basisvalue, ss), f_group(f_add([f_float(B), f_mul([str(symbol_y), f_float(A)])]))]),
+                                             f_mul([f_component(f_basisvalue, tt), f_float(C)])])
+                        code += [f_assign(assign_to, assign_from)]
 
             # FIAT_NEW code (loop 4 in FIAT).
             # for p in range(n+1):
             #    for q in range(n-p+1):
             #        results[idx(p,q),:] *= math.sqrt((p+0.5)*(p+q+1.0))
-            lines = []
-            loop_vars = [(str(symbol_p), 0, int_n1), \
-                         (str(symbol_q), 0, f_sub([int_n1, str(symbol_p)]))]
-            # Compute indices.
-            lines.append(f_assign(idx0, _idx2D(symbol_p, symbol_q)))
-            # Compute value.
-            fac0 = create_product([symbol_p + float_0_5, symbol_p + symbol_q + float_1])
-            fac2 = create_symbol( f_sqrt(str(fac0)), CONST )
-            lines += [f_imul(str(basis_idx0), fac2)]
-            # Create loop (block of lines).
-            code += f_loop(lines, loop_vars)
+            n1 = embedded_degree + 1
+            for r in range(0, n1):
+                for s in range(0, n1 - r):
+                    rr = _idx2d(r, s)
+                    A = (r + 0.5)*(r + s + 1)
+                    assign_to = f_component(f_basisvalue, rr)
+                    code += [f_imul(assign_to, f_sqrt(A))]
 
     # 3D
     elif (element_cell_domain == "tetrahedron"):
+
+        # FIAT_NEW code (compute index function) TetrahedronExpansionSet.
+        # def idx(p,q,r):
+        #     return (p+q+r)*(p+q+r+1)*(p+q+r+2)/6 + (q+r)*(q+r+1)/2 + r
+        def _idx3d(p, q, r):
+            return (p+q+r)*(p+q+r+1)*(p+q+r+2)/6 + (q+r)*(q+r+1)/2 + r
+
         # FIAT_NEW.expansions.TetrahedronExpansionSet.
 
         # Compute helper factors.
@@ -782,37 +792,33 @@ def _compute_basisvalues(data, dof_data):
                 #    a2 = p / (p + 1.0)
                 #    results[idx(p+1,0,0)] = a1 * factor1 * results[idx(p,0,0)] \
                 #        -a2 * factor2 * results[ idx(p-1,0,0) ]
-                lines = []
-                loop_vars = [(str(symbol_p), 1, embedded_degree)]
-                # Compute indices.
-                lines.append(f_assign(idx0, _idx3D(f_group(f_add([str(symbol_p), int_1])), int_0, int_0)))
-                lines.append(f_assign(idx1, _idx3D(str(symbol_p)               , int_0, int_0)))
-                lines.append(f_assign(idx2, _idx3D(f_group(f_sub([str(symbol_p), int_1])), int_0, int_0)))
-                # Compute value.
-                fac1 = create_fraction(float_2*symbol_p + float_1, symbol_p + float_1)
-                fac2 = create_fraction(symbol_p, symbol_p + float_1)
-                fac3 = create_product([fac1, f1, basis_idx1]) - create_product([fac2, f2, basis_idx2])
-                lines.append(f_assign(str(basis_idx0), fac3))
-                # Create loop (block of lines).
-                code += f_loop(lines, loop_vars)
+                for r in range(1, embedded_degree):
+                    rr = _idx3d((r + 1), 0, 0)
+                    ss = _idx3d(r, 0, 0)
+                    tt = _idx3d((r - 1), 0, 0)
+                    A = (2*r + 1.0)/(r + 1)
+                    B = r/(r + 1.0)
+                    assign_to = f_component(f_basisvalue, rr)
+                    assign_from = f_sub([f_mul([f_float(A), str(f1), f_component(f_basisvalue, ss)]), f_mul([f_float(B), str(f2), f_component(f_basisvalue, tt)])])
+                    code += [f_assign(assign_to, assign_from)]
 
             # FIAT_NEW code (loop 2 in FIAT).
             # q = 1
             # for p in range(0,n):
             #    results[idx(p,1,0)] = results[idx(p,0,0)] \
             #        * ( p * (1.0 + y) + ( 2.0 + 3.0 * y + z ) / 2 )
-            lines = []
-            loop_vars = [(str(symbol_p), 0, embedded_degree)]
-            # Compute indices.
-            lines.append(f_assign(idx0, _idx3D(symbol_p, int_1, int_0)))
-            lines.append(f_assign(idx1, _idx3D(symbol_p, int_0, int_0)))
-            # Compute value.
-            fac1 = create_fraction(float_2 + float_3*symbol_y + symbol_z, float_2)
-            fac2 = create_product([symbol_p, float_1 + symbol_y])
-            fac3 = create_product([basis_idx1, fac2 + fac1])
-            lines.append(f_assign(str(basis_idx0), fac3))
-            # Create loop (block of lines).
-            code += f_loop(lines, loop_vars)
+
+            for r in range(0, embedded_degree):
+                rr = _idx3d(r, 1, 0)
+                ss = _idx3d(r, 0, 0)
+                assign_to = f_component(f_basisvalue, rr)
+                term0 = f_mul([f_float(0.5), f_group(f_add([f_float(2), f_mul([f_float(3), str(symbol_y)]), str(symbol_z)]))])
+                if r == 0:
+                    assign_from = f_mul([term0, f_component(f_basisvalue, ss)])
+                else:
+                    term1 = f_mul([f_float(r), f_group(f_add([f_float(1), str(symbol_y)]))])
+                    assign_from = f_mul([f_group(f_add([term0, term1])), f_component(f_basisvalue, ss)])
+                code += [f_assign(assign_to, assign_from)]
 
             # Only active is embedded_degree > 1.
             if embedded_degree > 1:
@@ -824,23 +830,18 @@ def _compute_basisvalues(data, dof_data):
                 #        qm1coeff = cq * factor5
                 #        results[idx(p,q+1,0)] = qmcoeff * results[idx(p,q,0)] \
                 #            - qm1coeff * results[idx(p,q-1,0)]
-                lines = []
-                loop_vars = [(str(symbol_p), 0, int_nm1),\
-                             (str(symbol_q), 1, f_sub([int_n, str(symbol_p)]))]
-                # Compute indices.
-                lines.append(f_assign(idx0, _idx3D(symbol_p, f_group(f_add([str(symbol_q), int_1])), int_0)))
-                lines.append(f_assign(idx1, _idx3D(symbol_p, symbol_q                     , int_0)))
-                lines.append(f_assign(idx2, _idx3D(symbol_p, f_group(f_sub([str(symbol_q), int_1])), int_0)))
-                # Comute all helper variables.
-                jrc = _jrc(float_2*symbol_p + float_1, float_0, symbol_q)
-                lines.append(f_assign(str(an), jrc[0]))
-                lines.append(f_assign(str(bn), jrc[1]))
-                lines.append(f_assign(str(cn), jrc[2]))
-                # Compute value.
-                fac1 = create_product([an*f3 + bn*f4, basis_idx1]) - cn*f5*basis_idx2
-                lines.append(f_assign(str(basis_idx0), fac1))
-                # Create loop (block of lines).
-                code += f_loop(lines, loop_vars)
+
+                for r in range(0, embedded_degree - 1):
+                    for s in range(1, embedded_degree - r):
+                        rr = _idx3d(r, (s + 1), 0)
+                        ss = _idx3d(r, s, 0)
+                        tt = _idx3d(r, s - 1, 0)
+                        (A, B, C) = _jrc(2*r + 1, 0, s)
+                        assign_to = f_component(f_basisvalue, rr)
+                        term0 = f_mul([f_group(f_add([f_mul([f_float(A), str(f3)]), f_mul([f_float(B), str(f4)])])), f_component(f_basisvalue, ss)])
+                        term1 = f_mul([f_float(C), str(f5), f_component(f_basisvalue, tt)])
+                        assign_from = f_sub([term0, term1])
+                        code += [f_assign(assign_to, assign_from)]
 
             # FIAT_NEW code (loop 4 in FIAT).
             # now handle r=1
@@ -848,18 +849,14 @@ def _compute_basisvalues(data, dof_data):
             #    for q in range(n-p):
             #        results[idx(p,q,1)] = results[idx(p,q,0)] \
             #            * ( 1.0 + p + q + ( 2.0 + q + p ) * z )
-            lines = []
-            loop_vars = [(str(symbol_p), 0, int_n),\
-                         (str(symbol_q), 0, f_sub([int_n, str(symbol_p)]))]
-            # Compute indices.
-            lines.append(f_assign(idx0, _idx3D(symbol_p, symbol_q, int_1)))
-            lines.append(f_assign(idx1, _idx3D(symbol_p, symbol_q, int_0)))
-            # Compute value.
-            fac1 = create_product([float_2 + symbol_p + symbol_q, symbol_z])
-            fac2 = create_product([basis_idx1, float_1 + symbol_p + symbol_q + fac1])
-            lines.append(f_assign(str(basis_idx0), fac2))
-            # Create loop (block of lines).
-            code += f_loop(lines, loop_vars)
+            for r in range(0, embedded_degree):
+                for s in range(0, embedded_degree - r):
+                    rr = _idx3d(r, s, 1)
+                    ss = _idx3d(r, s, 0)
+                    assign_to = f_component(f_basisvalue, rr)
+                    A = f_add([f_mul([f_float(2 + r + s), str(symbol_z)]), f_float(1 + r + s)])
+                    assign_from = f_mul([f_group(A), f_component(f_basisvalue, ss)])
+                    code += [f_assign(assign_to, assign_from)]
 
             # Only active is embedded_degree > 1.
             if embedded_degree > 1:
@@ -872,44 +869,34 @@ def _compute_basisvalues(data, dof_data):
                 #             results[idx(p,q,r+1)] = \
                 #                         (ar * z + br) * results[idx(p,q,r) ] \
                 #                         - cr * results[idx(p,q,r-1) ]
-                lines = []
-                loop_vars = [(str(symbol_p), 0, int_nm1),\
-                             (str(symbol_q), 0, f_sub([int_nm1, str(symbol_p)])),\
-                             (str(symbol_r), 1, f_sub([int_n, str(symbol_p), str(symbol_q)]))]
-                # Compute indices.
-                lines.append(f_assign(idx0, _idx3D(symbol_p, symbol_q, f_add([str(symbol_r), int_1]))))
-                lines.append(f_assign(idx1, _idx3D(symbol_p, symbol_q, symbol_r)))
-                lines.append(f_assign(idx2, _idx3D(symbol_p, symbol_q, f_sub([str(symbol_r), int_1]))))
-                # Comute all helper variables.
-                jrc = _jrc(float_2*symbol_p + float_2*symbol_q + float_2, float_0, symbol_r)
-                lines.append(f_assign(str(an), jrc[0]))
-                lines.append(f_assign(str(bn), jrc[1]))
-                lines.append(f_assign(str(cn), jrc[2]))
-                # Compute value.
-                fac1 = create_product([an*symbol_z + bn, basis_idx1]) - cn*basis_idx2
-                lines.append(f_assign(str(basis_idx0), fac1))
-                # Create loop (block of lines).
-                code += f_loop(lines, loop_vars)
+                for r in range(embedded_degree - 1):
+                    for s in range(0, embedded_degree - r - 1):
+                        for t in range(1, embedded_degree - r - s):
+                            rr = _idx3d(r, s, ( t + 1))
+                            ss = _idx3d(r, s, t)
+                            tt = _idx3d(r, s, t - 1)
+
+                            (A, B, C) = _jrc(2*r + 2*s + 2, 0, t)
+                            assign_to = f_component(f_basisvalue, rr)
+                            az_b = f_group(f_add([f_float(B), f_mul([f_float(A), str(symbol_z)])]))
+                            assign_from = f_sub([f_mul([f_component(f_basisvalue, ss), az_b]), f_mul([f_float(C), f_component(f_basisvalue, tt)])])
+                            code += [f_assign(assign_to, assign_from)]
 
             # FIAT_NEW code (loop 6 in FIAT).
             # for p in range(n+1):
             #    for q in range(n-p+1):
             #        for r in range(n-p-q+1):
             #            results[idx(p,q,r)] *= math.sqrt((p+0.5)*(p+q+1.0)*(p+q+r+1.5))
-            lines = []
-            loop_vars = [(str(symbol_p), 0, int_n1),\
-                         (str(symbol_q), 0, f_sub([int_n1, str(symbol_p)])),\
-                         (str(symbol_r), 0, f_sub([int_n1, str(symbol_p), str(symbol_q)]))]
-            # Compute indices.
-            lines.append(f_assign(idx0, _idx3D(symbol_p, symbol_q, symbol_r)))
-            # Compute value
-            fac0 = create_product([symbol_p + float_0_5,\
-                                   symbol_p + symbol_q + float_1,\
-                                   symbol_p + symbol_q + symbol_r + float_1_5])
-            fac2 = create_symbol( f_sqrt(str(fac0)), CONST )
-            lines += [format["imul"](str(basis_idx0), fac2)]
-            # Create loop (block of lines).
-            code += f_loop(lines, loop_vars)
+            for r in range(embedded_degree + 1):
+                for s in range(embedded_degree - r + 1):
+                    for t in range(embedded_degree - r - s + 1):
+                        rr = _idx3d(r, s, t)
+                        A = (r + 0.5)*(r + s + 1)*(r + s + t + 1.5)
+                        assign_to = f_component(f_basisvalue, rr)
+                        multiply_by = f_sqrt(A)
+                        myline = f_imul(assign_to, multiply_by)
+                        code += [myline]
+
     else:
         error("Cannot compute basis values for shape: %d" % elemet_cell_domain)
 
