@@ -12,11 +12,11 @@ It uses Instant to wrap the generated code into a Python module."""
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Johan Hake, 2008-2009
 # Modified by Ilmar Wilbers, 2008
@@ -57,20 +57,20 @@ FFC_PARAMETERS_JIT["no-evaluate_basis_derivatives"] = True
 # Set debug level for Instant
 instant.set_logging_level("warning")
 
-def jit(object, parameters=None, common_cell=None):
+def jit(ufl_object, parameters=None, common_cell=None):
     """Just-in-time compile the given form or element
 
     Parameters:
 
-      object     : The object to be compiled
+      ufl_object : The UFL object to be compiled
       parameters : A set of parameters
     """
 
     # Check if we get an element or a form
-    if isinstance(object, FiniteElementBase):
-        return jit_element(object, parameters)
+    if isinstance(ufl_object, FiniteElementBase):
+        return jit_element(ufl_object, parameters)
     else:
-        return jit_form(object, parameters, common_cell)
+        return jit_form(ufl_object, parameters, common_cell)
 
 def jit_form(form, parameters=None, common_cell=None):
     "Just-in-time compile the given form."
@@ -145,9 +145,11 @@ def jit_form(form, parameters=None, common_cell=None):
             cache_dir = cache_dir)
 
         # Remove code
-        os.unlink(hfile)
+        if os.path.isfile(hfile):
+            os.unlink(hfile)
         if parameters["split"] :
-            os.unlink(cppfile)
+            if os.path.isfile(cppfile):
+                os.unlink(cppfile)
 
         # Extract compiled form
         compiled_form = _extract_form(module, prefix)
