@@ -1,6 +1,6 @@
 "QuadratureTransformer (optimised) for quadrature code generation to translate UFL expressions."
 
-# Copyright (C) 2009-2010 Kristian B. Oelgaard
+# Copyright (C) 2009-2011 Kristian B. Oelgaard
 #
 # This file is part of FFC.
 #
@@ -11,16 +11,16 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2009
 #
 # First added:  2009-03-18
-# Last changed: 2010-03-11
+# Last changed: 2011-11-22
 
 # Python modules.
 from numpy import shape
@@ -86,6 +86,9 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
                 code[key] = val[0]
             else:
                 error("Where did the values go?")
+            # If value is zero just ignore it.
+            if abs(code[key].val) < format["epsilon"]:
+                del code[key]
 
         return code
 
@@ -97,18 +100,20 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
         # Sort operands in objects that needs permutation and objects that does not.
         for op in operands:
+            # If we get an empty dict, something was zero and so is the product.
+            if not op:
+                return {}
             if len(op) > 1 or (op and op.keys()[0] != ()):
                 permute.append(op)
-            elif op:
+            elif op and op.keys()[0] == ():
                 not_permute.append(op[()])
 
         # Create permutations.
         # TODO: After all indices have been expanded I don't think that we'll
         # ever get more than a list of entries and values.
-        permutations = create_permutations(permute)
-
         #print("\npermute: " + repr(permute))
         #print("\nnot_permute: " + repr(not_permute))
+        permutations = create_permutations(permute)
         #print("\npermutations: " + repr(permutations))
 
         # Create code.
@@ -171,16 +176,14 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 #            sym = create_symbol(format["std power"](str(val), exp), val.t)
 #            sym.base_expr = val
 #            sym.base_op = 1 # Add one operation for the pow() function.
-            sym = create_symbol(format["std power"], val.t, val, 1)
-            sym.exp = exp
+            sym = create_symbol(format["std power"], val.t, val, 1, exp)
             return {(): sym}
         elif isinstance(expo, (Coefficient, Operator)):
-            exp = self.visit(expo)
+            exp = self.visit(expo)[()]
 #            sym = create_symbol(format["std power"](str(val), exp[()]), val.t)
 #            sym.base_expr = val
 #            sym.base_op = 1 # Add one operation for the pow() function.
-            sym = create_symbol(format["std power"], val.t, val, 1)
-            sym.exp = exp[()]
+            sym = create_symbol(format["std power"], val.t, val, 1, exp)
             return {(): sym}
         else:
             error("power does not support this exponent: " + repr(expo))
@@ -203,7 +206,17 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
     # -------------------------------------------------------------------------
     # Condition, Conditional (conditional.py).
     # -------------------------------------------------------------------------
-    def condition(self, o, *operands):
+    def not_condition(self, o, *operands):
+        # This is a Condition but not a BinaryCondition, and the operand will be another Condition
+        # Get condition expression and do safety checks.
+        # Might be a bit too strict?
+        c, = operands
+        ffc_assert(len(c) == 1 and c.keys()[0] == (),\
+            "Condition for NotCondition should only be one function: " + repr(c))
+        sym = create_symbol("", c[()].t, cond=(c[()], format["not"]))
+        return {(): sym}
+
+    def binary_condition(self, o, *operands):
 
         # Get LHS and RHS expressions and do safety checks.
         # Might be a bit too strict?
@@ -216,7 +229,8 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         # Map names from UFL to cpp.py.
         name_map = {"==":"is equal", "!=":"not equal",\
                     "<":"less than", ">":"greater than",\
-                    "<=":"less equal", ">=":"greater equal"}
+                    "<=":"less equal", ">=":"greater equal",\
+                    "&&": "and", "||": "or"}
 
         # Get the minimum type
         t = min(lhs[()].t, rhs[()].t)
@@ -256,7 +270,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         return {():name}
 
     # -------------------------------------------------------------------------
-    # FacetNormal, CellVolume (geometry.py).
+    # FacetNormal, CellVolume, Circumradius, FacetArea (geometry.py).
     # -------------------------------------------------------------------------
     def facet_normal(self, o,  *operands):
         #print("Visiting FacetNormal:")
@@ -309,6 +323,22 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
 
         return {():create_symbol(circumradius, GEO)}
 
+    def facet_area(self, o,  *operands):
+        # Safety check.
+        ffc_assert(not operands, "Didn't expect any operands for FacetArea: " + repr(operands))
+
+        # FIXME: KBO: This has to change for higher order elements
+        # NOTE: Omitting restriction because the area of a facet is the same
+        # on both sides.
+        # FIXME: Since we use the scale factor, facet area has no meaning
+        # for cell integrals. (Need check in FFC or UFL).
+        area = format["facet area"]
+        self.trans_set.add(area)
+
+        return {():create_symbol(area, GEO)}
+
+    # -------------------------------------------------------------------------
+
     def create_argument(self, ufl_argument, derivatives, component, local_comp,
                   local_offset, ffc_element, transformation, multiindices):
         "Create code for basis functions, and update relevant tables of used basis."
@@ -476,6 +506,26 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
             operand[key] = new_val
         return operand
 
+    def _bessel_function(self, operands, format_function):
+        # TODO: Are these safety checks needed?
+        # TODO: work on reference instead of copies? (like math_function)
+        ffc_assert(len(operands) == 2,\
+          "BesselFunctions expect two operands of function type: " + repr(operands))
+        nu, x = operands
+        ffc_assert(len(nu) == 1 and () in nu,\
+          "Expecting one operand of function type as first argument to BesselFunction : " + repr(nu))
+        ffc_assert(len(x) == 1 and () in x,\
+          "Expecting one operand of function type as second argument to BesselFunction : " + repr(x))
+        nu = nu[()]
+        x = x[()]
+        if nu is None:
+            nu = format["floating point"](0.0)
+        if x is None:
+            x = format["floating point"](0.0)
+
+        sym = create_symbol(format_function, x.t, x, 1, nu)
+        return {():sym}
+
     # -------------------------------------------------------------------------
     # Helper functions for code_generation()
     # -------------------------------------------------------------------------
