@@ -11,16 +11,16 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Mehdi Nikbakht, 2010
 #
 # First added:  2009-01-07
-# Last changed: 2011-02-21
+# Last changed: 2011-11-28
 
 # Python modules.
 import numpy
@@ -52,6 +52,7 @@ def generate_integral_code(ir, prefix, parameters):
     code["destructor"] = format["do nothing"]
     code["tabulate_tensor"] = _tabulate_tensor(ir, parameters)
     code["tabulate_tensor_quadrature"] = format["exception"](tabulate_tensor_quadrature_error)
+    code["additional_includes_set"] = ir["additional_includes_set"]
 
     return code
 
@@ -159,6 +160,7 @@ def _tabulate_tensor(ir, parameters):
     # Add common (for cell, exterior and interior) geo code.
     jacobi_code += "\n\n" + format["generate cell volume"](geo_dim, domain_type)
     jacobi_code += "\n\n" + format["generate circumradius"](geo_dim, domain_type)
+    jacobi_code += "\n\n" + format["generate facet area"](geo_dim)
 
     # After we have generated the element code for all facets we can remove
     # the unused transformations and tabulate the used psi tables and weights.
@@ -209,7 +211,8 @@ def _generate_element_tensor(integrals, sets, optimise_parameters):
     f_ip      = format["integration points"]
     f_I       = format["ip constant"]
     f_loop    = format["generate loop"]
-    f_coords  = format["generate ip coordinates"]
+    f_ip_coords  = format["generate ip coordinates"]
+    f_coords     =  format["coordinates"]
     f_double  = format["float declaration"]
     f_decl    = format["declaration"]
     f_X       = format["ip coordinates"]
@@ -240,12 +243,13 @@ def _generate_element_tensor(integrals, sets, optimise_parameters):
             name, geo_dim, ip, r = coordinate
             element_code += ["", f_comment("Declare array to hold physical coordinate of quadrature point.")]
             element_code += [f_decl(f_double, f_X(points, geo_dim))]
-            ops, coord_code = f_coords(geo_dim, points, name, ip, r)
+            ops, coord_code = f_ip_coords(geo_dim, points, name, ip, r)
             ip_code += ["", f_comment("Compute physical coordinate of quadrature point, operations: %d." % ops)]
             ip_code += [coord_code]
             num_ops += ops
-            # Update used psi tables.
+            # Update used psi tables and transformation set.
             sets[1].add(name)
+            sets[3].add(f_coords(r))
 
         # Generate code to compute function values.
         if functions:
@@ -383,7 +387,6 @@ def _generate_integral_code(points, terms, sets, optimise_parameters):
 
     # Loop terms and create code.
     for loop, (data, entry_vals) in terms.items():
-
         # If we don't have any entry values, there's no need to generate the
         # loop.
         if not entry_vals:
@@ -527,8 +530,12 @@ def _tabulate_psis(tables, used_psi_tables, inv_name_map, used_nzcs, optimise_pa
 
     # Get list of non zero columns, if we ignore ones, ignore columns with one component.
     if optimise_parameters["ignore ones"]:
-        nzcs = [val[1] for key, val in inv_name_map.items()\
-                                        if val[1] and len(val[1][1]) > 1]
+        nzcs = []
+        for key, val in inv_name_map.items():
+            # Check if we have a table of ones or if number of non-zero columns
+            # is larger than one.
+            if val[1] and len(val[1][1]) > 1 or not val[3]:
+                nzcs.append(val[1])
     else:
         nzcs = [val[1] for key, val in inv_name_map.items()\
                                         if val[1]]
