@@ -9,11 +9,11 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2010-02-08
 # Last changed: 2011-01-11
@@ -65,11 +65,19 @@ def optimize_integral_ir(ir, parameters):
 
 def _simplify_expression(integral, geo_consts, psi_tables_map):
     for points, terms, functions, ip_consts, coordinate, conditionals in integral:
-        for loop, (data, entry_vals) in terms.iteritems():
+        # NOTE: sorted is needed to pass the regression tests on the buildbots
+        # but it might be inefficient for speed.
+        # A solution could be to only compare the output of evaluating the
+        # integral, not the header files.
+        for loop, (data, entry_vals) in sorted(terms.iteritems()):
             t_set, u_weights, u_psi_tables, u_nzcs, basis_consts = data
             new_entry_vals = []
             psi_tables = set()
-            for entry, val, ops in entry_vals:
+            # NOTE: sorted is needed to pass the regression tests on the buildbots
+            # but it might be inefficient for speed.
+            # A solution could be to only compare the output of evaluating the
+            # integral, not the header files.
+            for entry, val, ops in sorted(entry_vals):
                 value = optimise_code(val, ip_consts, geo_consts, t_set)
                 # Check if value is zero
                 if value.val:
@@ -102,7 +110,7 @@ def _extract_variables(val, basis_consts, ip_consts, geo_consts, t_set, optimisa
         if val.base_expr is None:
             return val
         new_base = _extract_variables(val.base_expr, basis_consts, ip_consts, geo_consts, t_set, optimisation)
-        new_sym = create_symbol(val.v, val.t, new_base, val.base_op)
+        new_sym = create_symbol(val.v, val.t, new_base, val.base_op, val.exp, val.cond)
         new_sym.exp = val.exp
         if new_sym.t == BASIS:
             return _reduce_expression(new_sym, [], basis_consts, f_B, True)
