@@ -1,6 +1,6 @@
 "QuadratureTransformer for quadrature code generation to translate UFL expressions."
 
-# Copyright (C) 2009-2010 Kristian B. Oelgaard
+# Copyright (C) 2009-2011 Kristian B. Oelgaard
 #
 # This file is part of FFC.
 #
@@ -11,17 +11,17 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Peter Brune, 2009
 # Modified by Anders Logg, 2009
 #
 # First added:  2009-02-09
-# Last changed: 2010-03-11
+# Last changed: 2011-11-22
 
 # Python modules.
 from numpy import shape
@@ -100,7 +100,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
                         continue
                     duplications[val] = 1
 
-                # Add a product for eacht term that has duplicate code
+                # Add a product for each term that has duplicate code
                 expressions = []
                 for expr, num_occur in duplications.items():
                     if num_occur > 1:
@@ -116,9 +116,11 @@ class QuadratureTransformer(QuadratureTransformerBase):
                     continue
                 code[key] = expressions[0]
             else:
-                # Check for zero valued sum
+                # Check for zero valued sum and delete from code
+                # This might result in returning an empty dict, but that should
+                # be interpreted as zero by other handlers.
                 if not value:
-                    code[key] = None
+                    del code[key]
                     continue
                 code[key] = value[0]
 
@@ -134,16 +136,18 @@ class QuadratureTransformer(QuadratureTransformerBase):
 
         # Sort operands in objects that needs permutation and objects that does not.
         for op in operands:
+            # If we get an empty dict, something was zero and so is the product.
+            if not op:
+                return {}
             if len(op) > 1 or (op and op.keys()[0] != ()):
                 permute.append(op)
-            elif op:
+            elif op and op.keys()[0] == ():
                 not_permute.append(op[()])
 
         # Create permutations.
-        permutations = create_permutations(permute)
-
         #print("\npermute: " + repr(permute))
         #print("\nnot_permute: " + repr(not_permute))
+        permutations = create_permutations(permute)
         #print("\npermutations: " + repr(permutations))
 
         # Create code.
@@ -195,15 +199,15 @@ class QuadratureTransformer(QuadratureTransformerBase):
                     pass
                 else:
                     value.append(v)
-            if value == []:
-                value = ["1"]
-
-            code[()] = f_mult(value)
-
+            # We did have values, but they might have been all ones.
+            if value == [] and not_permute != []:
+                code[()] = f_mult(["1"])
+            else:
+                code[()] = f_mult(value)
         return code
 
     def division(self, o, *operands):
-        #print("\n\nVisiting Division: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        #print("Visiting Division with operands: \n" + "\n".join(map(repr,operands)))
 
         # Prefetch formats to speed up code generation.
         f_div      = format["div"]
@@ -279,20 +283,30 @@ class QuadratureTransformer(QuadratureTransformerBase):
     # -------------------------------------------------------------------------
     # Condition, Conditional (conditional.py).
     # -------------------------------------------------------------------------
-    def condition(self, o, *operands):
+    def not_condition(self, o, *operands):
+        # This is a Condition but not a BinaryCondition, and the operand will be another Condition
+        # Get condition expression and do safety checks.
+        # Might be a bit too strict?
+        cond, = operands
+        ffc_assert(len(cond) == 1 and cond.keys()[0] == (),\
+            "Condition for NotCondition should only be one function: " + repr(cond))
+        return {(): format["not"](cond[()])}
+
+    def binary_condition(self, o, *operands):
 
         # Get LHS and RHS expressions and do safety checks.
         # Might be a bit too strict?
         lhs, rhs = operands
         ffc_assert(len(lhs) == 1 and lhs.keys()[0] == (),\
-            "LHS of Condtion should only be one function: " + repr(lhs))
+            "LHS of Condition should only be one function: " + repr(lhs))
         ffc_assert(len(rhs) == 1 and rhs.keys()[0] == (),\
-            "RHS of Condtion should only be one function: " + repr(rhs))
+            "RHS of Condition should only be one function: " + repr(rhs))
 
         # Map names from UFL to cpp.py.
         name_map = {"==":"is equal", "!=":"not equal",\
                     "<":"less than", ">":"greater than",\
-                    "<=":"less equal", ">=":"greater equal"}
+                    "<=":"less equal", ">=":"greater equal",\
+                    "&&":"and", "||": "or"}
 
         # Get values and test for None
         l_val = lhs[()]
@@ -335,7 +349,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
         return {():name}
 
     # -------------------------------------------------------------------------
-    # FacetNormal, CellVolume (geometry.py).
+    # FacetNormal, CellVolume, Circumradius, FacetArea (geometry.py).
     # -------------------------------------------------------------------------
     def facet_normal(self, o,  *operands):
         #print("Visiting FacetNormal:")
@@ -384,6 +398,21 @@ class QuadratureTransformer(QuadratureTransformerBase):
 
         return {():circumradius}
 
+    def facet_area(self, o,  *operands):
+        # Safety check.
+        ffc_assert(not operands, "Didn't expect any operands for FacetArea: " + repr(operands))
+
+        # FIXME: KBO: This has to change for higher order elements
+        # NOTE: Omitting restriction because the area of a facet is the same
+        # on both sides.
+        # FIXME: Since we use the scale factor, facet area has no meaning
+        # for cell integrals. (Need check in FFC or UFL).
+        area = format["facet area"]
+        self.trans_set.add(area)
+
+        return {():area}
+
+    # -------------------------------------------------------------------------
 
     def create_argument(self, ufl_argument, derivatives, component, local_comp,
                   local_offset, ffc_element, transformation, multiindices):
@@ -580,6 +609,28 @@ class QuadratureTransformer(QuadratureTransformerBase):
             new_operand[key] = format_function(val)
         return new_operand
 
+    def _bessel_function(self, operands, format_function):
+        # TODO: Are these safety checks needed?
+        ffc_assert(len(operands) == 2,\
+          "BesselFunctions expect two operands of function type: " + repr(operands))
+        nu, x = operands
+        ffc_assert(len(nu) == 1 and () in nu,\
+          "Expecting one operand of function type as first argument to BesselFunction : " + repr(nu))
+        ffc_assert(len(x) == 1 and () in x,\
+          "Expecting one operand of function type as second argument to BesselFunction : " + repr(x))
+        nu = nu[()]
+        x = x[()]
+        if nu is None:
+            nu = format["floating point"](0.0)
+        if x is None:
+            x = format["floating point"](0.0)
+
+        # Use format function on arguments.
+        # NOTE: Order of nu and x is reversed compared to the UFL and C++
+        # function calls because of how Symbol treats exponents.
+        # this will change once quadrature optimisations has been cleaned up.
+        return {():format_function(x, nu)}
+
     # -------------------------------------------------------------------------
     # Helper functions for code_generation()
     # -------------------------------------------------------------------------
