@@ -1,6 +1,6 @@
 "QuadratureTransformerBase, a common class for quadrature transformers to translate UFL expressions."
 
-# Copyright (C) 2009-2010 Kristian B. Oelgaard
+# Copyright (C) 2009-2011 Kristian B. Oelgaard
 #
 # This file is part of FFC.
 #
@@ -11,14 +11,14 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2009-10-13
-# Last changed: 2010-03-11
+# Last changed: 2011-10-20
 
 # Python modules.
 from itertools import izip
@@ -28,9 +28,11 @@ from numpy import shape, array
 # UFL Classes.
 from ufl.classes import MultiIndex, FixedIndex, Index
 from ufl.common import StackDict, Stack
+from ufl.permutation import build_component_numbering
 
 # UFL Algorithms.
-from ufl.algorithms import propagate_restrictions, Transformer, tree_format, strip_variables
+from ufl.algorithms import propagate_restrictions, Transformer, tree_format
+from ufl.algorithms import strip_variables
 
 # FFC modules.
 from ffc.log import ffc_assert, error, info
@@ -78,6 +80,7 @@ class QuadratureTransformerBase(Transformer):
         self.restriction = None
         self.coordinate = None
         self.conditionals = {}
+        self.additional_includes_set = set()
 
         # Stacks.
         self._derivatives = []
@@ -203,13 +206,9 @@ class QuadratureTransformerBase(Transformer):
     # condition.py, constantvalue.py, function.py, geometry.py, lifting.py,
     # mathfunctions.py, restriction.py
     # -------------------------------------------------------------------------
-#    def condition(self, o):
-#        print "\n\nVisiting Condition:", repr(o)
-#        error("Condition is not supported (yet).")
-
-#    def conditional(self, o):
-#        print "\n\nVisiting Condition:", repr(o)
-#        error("Conditional is not supported (yet).")
+    def condition(self, o):
+        print "\n\nVisiting Condition:", repr(o)
+        error("This type of Condition is not supported (yet).")
 
     def constant_value(self, o):
         print "\n\nVisiting ConstantValue:", repr(o)
@@ -239,6 +238,10 @@ class QuadratureTransformerBase(Transformer):
         print "\n\nVisiting MathFunction:", repr(o)
         error("This MathFunction is not supported (yet).")
 
+    def bessel_function(self, o):
+        print "\n\nVisiting BesselFunction:", repr(o)
+        error("BesselFunction is not implemented (yet).")
+
     def restricted(self, o):
         print "\n\nVisiting Restricted:", repr(o)
         error("This type of Restricted is not supported (only positive and negative are currently supported).")
@@ -482,10 +485,18 @@ class QuadratureTransformerBase(Transformer):
 
         # Safety checks.
         ffc_assert(not operands, "Didn't expect any operands for FacetNormal: " + repr(operands))
-        ffc_assert(len(components) == 1, " expects 1 component index: " + repr(components))
+
+        if components == (): # 1D case
+            ffc_assert(o.cell().geometric_dimension() == 1,
+                       "Expecting component when not in 1D.")
+            c = 0
+        else:
+            ffc_assert(len(components) == 1,
+                       " expects 1 component index: " + repr(components))
+            c, = components
 
         # Generate the appropriate coordinate and update tables.
-        coordinate = format["ip coordinates"](self.points, components[0])
+        coordinate = format["ip coordinates"](self.points, c)
         self._generate_affine_map()
 
         return self._create_symbol(coordinate, IP)
@@ -585,6 +596,34 @@ class QuadratureTransformerBase(Transformer):
         #print("\n\nVisiting Atan: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
         return self._math_function(operands, format["atan"])
 
+    def erf(self, o, *operands):
+        #print("\n\nVisiting Erf: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+        return self._math_function(operands, format["erf"])
+
+    def bessel_i(self, o, *operands):
+        #print("\n\nVisiting Bessel_I: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+#        self.additional_includes_set.add("#include <tr1/cmath>")
+        self.additional_includes_set.add("#include <boost/math/tr1.hpp>")
+        return self._bessel_function(operands, format["bessel_i"])
+
+    def bessel_j(self, o, *operands):
+        #print("\n\nVisiting Bessel_J: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+#        self.additional_includes_set.add("#include <tr1/cmath>")
+        self.additional_includes_set.add("#include <boost/math/tr1.hpp>")
+        return self._bessel_function(operands, format["bessel_j"])
+
+    def bessel_k(self, o, *operands):
+        #print("\n\nVisiting Bessel_K: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+#        self.additional_includes_set.add("#include <tr1/cmath>")
+        self.additional_includes_set.add("#include <boost/math/tr1.hpp>")
+        return self._bessel_function(operands, format["bessel_k"])
+
+    def bessel_y(self, o, *operands):
+        #print("\n\nVisiting Bessel_Y: " + repr(o) + "with operands: " + "\n".join(map(repr,operands)))
+#        self.additional_includes_set.add("#include <tr1/cmath>")
+        self.additional_includes_set.add("#include <boost/math/tr1.hpp>")
+        return self._bessel_function(operands, format["bessel_y"])
+
     # -------------------------------------------------------------------------
     # PositiveRestricted and NegativeRestricted (restriction.py).
     # -------------------------------------------------------------------------
@@ -685,6 +724,8 @@ class QuadratureTransformerBase(Transformer):
     # -------------------------------------------------------------------------
     def generate_terms(self, integrand):
         "Generate terms for code generation."
+        #print integrand
+        #print tree_format(integrand, 0, False)
         # Get terms.
         terms = self.visit(integrand)
 
@@ -704,7 +745,7 @@ class QuadratureTransformerBase(Transformer):
             sets.append(used_nzcs)
 
             # Create loop information and entry from key info and insert into dict.
-            loop, entry = self._create_loop_entry(key)
+            loop, entry = self._create_loop_entry(key, f_nzc)
             if not loop in new_terms:
                 sets.append({})
                 new_terms[loop] = [sets, [(entry, value, ops)]]
@@ -714,7 +755,7 @@ class QuadratureTransformerBase(Transformer):
                 new_terms[loop][1].append((entry, value, ops))
         return new_terms
 
-    def _create_loop_entry(self, key):
+    def _create_loop_entry(self, key, f_nzc):
 
         # TODO: Verify that test and trial functions will ALWAYS be rearranged to 0 and 1.
         indices = {-2: format["first free index"], -1: format["second free index"],
@@ -734,7 +775,7 @@ class QuadratureTransformerBase(Transformer):
                         "Linear forms must be defined using test functions only: " + repr(key))
             index_j, entry, range_j, space_dim_j = key
             loop = ((indices[index_j], 0, range_j),)
-            if range_j == 1 and self.optimise_parameters["ignore ones"]:
+            if range_j == 1 and self.optimise_parameters["ignore ones"] and not (f_nzc in entry):
                 loop = ()
         elif len(key) == 2:
             # Extract test and trial loops in correct order and check if for is legal.
@@ -750,9 +791,9 @@ class QuadratureTransformerBase(Transformer):
             index_k, entry_k, range_k, space_dim_k = key1
 
             loop = []
-            if not (range_j == 1 and self.optimise_parameters["ignore ones"]):
+            if not (range_j == 1 and self.optimise_parameters["ignore ones"]) or f_nzc in entry_j:
                 loop.append((indices[index_j], 0, range_j))
-            if not (range_k == 1 and self.optimise_parameters["ignore ones"]):
+            if not (range_k == 1 and self.optimise_parameters["ignore ones"]) or f_nzc in entry_k:
                 loop.append((indices[index_k], 0, range_k))
             entry = format["add"]([format["mul"]([entry_j, str(space_dim_k)]), entry_k])
             loop = tuple(loop)
@@ -787,8 +828,11 @@ class QuadratureTransformerBase(Transformer):
                                  derivatives):
         "Helper function for both Coefficient and Argument."
 
+        # Get UFL element.
+        ufl_element = ufl_function.element()
+
         # Get local component (in case we have mixed elements).
-        local_comp, local_elem = ufl_function.element().extract_component(component)
+        local_comp, local_elem = ufl_element.extract_component(component)
 
         # Check that we don't take derivatives of QuadratureElements.
         quad_element = local_elem.family() == "Quadrature"
@@ -796,10 +840,10 @@ class QuadratureTransformerBase(Transformer):
                    "Derivatives of Quadrature elements are not supported: " + repr(ufl_function))
 
         # Create FFC element.
-        ffc_element = create_element(ufl_function.element())
+        ffc_element = create_element(ufl_element)
 
         # Get relevant sub element and mapping.
-        sub_element = create_element(ufl_function.element().extract_component(tuple(component))[1])
+        sub_element = create_element(local_elem)
 
         # Assuming that mappings for all basisfunctions are equal (they should be).
         transformation = sub_element.mapping()[0]
@@ -812,12 +856,12 @@ class QuadratureTransformerBase(Transformer):
         else:
             local_comp = 0
 
-        # Map component using UFL map
-        # NOTE: We rely implicitly on a similar ordering of sub elements of mixed elements in UFL and FFC.
-        if len(component) > 1:
-            component = ufl_function.element()._sub_element_mapping[tuple(component)]
-        elif component:
-            component = component[0]
+        # Check that component != not () since the UFL component map will turn
+        # it into 0, and () does not mean zeroth component in this context.
+        if component != ():
+            # Map component using component map from UFL.
+            comp_map, comp_num = build_component_numbering(ufl_element.value_shape(), ufl_element.symmetry())
+            component = comp_map[component]
 
         # Compute the local offset (needed for non-affine mappings).
         local_offset = 0
@@ -1054,7 +1098,7 @@ class QuadratureTransformerBase(Transformer):
                 vals.append(affine_map[len(p)](p))
             self.unique_tables[name] = array(vals)
         if self.coordinate is None:
-            ip = 1
+            ip = 0
             r = None
             if num_ip > 1:
                 ip = f_ip
