@@ -1,6 +1,6 @@
 "This file implements a class to represent a symbol."
 
-# Copyright (C) 2009-2010 Kristian B. Oelgaard
+# Copyright (C) 2009-2011 Kristian B. Oelgaard
 #
 # This file is part of FFC.
 #
@@ -11,14 +11,14 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2009-07-12
-# Last changed: 2010-03-11
+# Last changed: 2011-06-28
 
 # FFC modules.
 from ffc.log import error
@@ -33,7 +33,7 @@ from expr import Expr
 
 class Symbol(Expr):
     __slots__ = ("v", "base_expr", "base_op", "exp", "cond")
-    def __init__(self, variable, symbol_type, base_expr=None, base_op=0, cond=()):
+    def __init__(self, variable, symbol_type, base_expr=None, base_op=0, expo=None, cond=()):
         """Initialise a Symbols object, it derives from Expr and contains
         the additional variables:
 
@@ -56,7 +56,7 @@ class Symbol(Expr):
         # ops = base_expr.ops() + base_ops = 2 + 1 = 3
         self.base_expr = base_expr
         self.base_op = base_op
-        self.exp = None
+        self.exp = expo
         self.cond = cond
 
         # If type of the base_expr is lower than the given symbol_type change type.
@@ -68,10 +68,15 @@ class Symbol(Expr):
         # Compute the representation now, such that we can use it directly
         # in the __eq__ and __ne__ methods (improves performance a bit, but
         # only when objects are cached).
-        if self.base_expr:
-            self._repr = "Symbol('%s', %s, %s, %d)" % (self.v, type_to_string[self.t], self.base_expr._repr, self.base_op)
+        if self.base_expr and self.exp is None:
+            self._repr = "Symbol('%s', %s, %s, %d)" % (self.v, type_to_string[self.t],\
+                         self.base_expr._repr, self.base_op)
+        elif self.base_expr:
+            self._repr = "Symbol('%s', %s, %s, %d, %s)" % (self.v, type_to_string[self.t],\
+                         self.base_expr._repr, self.base_op, self.exp)
         elif self.cond:
-            self._repr = "Symbol('%s', %s, %s, %d, %s)" % (self.v, type_to_string[self.t], self.base_expr, self.base_op, self.cond)
+            self._repr = "Symbol('%s', %s, %s, %d, %s, %s)" % (self.v, type_to_string[self.t],\
+                          self.base_expr, self.base_op, self.exp, self.cond)
         else:
             self._repr = "Symbol('%s', %s)" % (self.v, type_to_string[self.t])
 
@@ -85,6 +90,8 @@ class Symbol(Expr):
             if self.cond == ():
                 return self.v
             else:
+                if len(self.cond) == 2:
+                    return self.cond[1](str(self.cond[0]))
                 return "".join([str(c) for c in self.cond])
         elif self.exp is None:
             return self.v(str(self.base_expr))
@@ -225,6 +232,8 @@ class Symbol(Expr):
         if self.base_expr:
             return self.base_op + self.base_expr.ops()
         elif self.cond:
+            if len(self.cond) == 2:
+                return self.base_op + self.cond[0].ops() + 1
             return self.base_op + self.cond[0].ops() + self.cond[2].ops() + 1
         return self.base_op
 
