@@ -23,11 +23,11 @@ in the intermediate representation under the key "foo".
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Marie E. Rognes 2010
 # Modified by Kristian B. Oelgaard 2010
@@ -56,7 +56,7 @@ from ffc import tensor
 
 not_implemented = None
 
-def compute_ir(analysis, parameters):
+def compute_ir(analysis, parameters, common_cell=None):
     "Compute intermediate representation."
 
     begin("Compiler stage 2: Computing intermediate representation")
@@ -79,7 +79,7 @@ def compute_ir(analysis, parameters):
 
     # Compute and flatten representation of integrals
     info("Computing representation of integrals")
-    irs = [_compute_integral_ir(fd, i, parameters) \
+    irs = [_compute_integral_ir(fd, i, parameters, common_cell) \
                for (i, fd) in enumerate(form_datas)]
     ir_integrals = [ir for ir in chain(*irs) if not ir is None]
 
@@ -136,7 +136,7 @@ def _compute_dofmap_ir(ufl_element, element_id, element_numbers):
 
     # Compute data for each function
     ir["signature"] = "FFC dofmap for " + repr(ufl_element)
-    ir["needs_mesh_entities"] = [d > 0 for d in num_dofs_per_entity]
+    ir["needs_mesh_entities"] = _needs_mesh_entities(element)
     ir["init_mesh"] = _init_mesh(element)
     ir["init_cell"] = None
     ir["init_cell_finalize"] = None
@@ -176,9 +176,18 @@ def _init_mesh(element):
     element = MixedElement(elements)
     return (_num_dofs_per_entity(element), num_reals)
 
-    # return _num_dofs_per_entity(element)
 
-def _compute_integral_ir(form_data, form_id, parameters):
+def _needs_mesh_entities(element):
+
+    # Note: The dof map for Real elements does not depend on the mesh 
+
+    num_dofs_per_entity = _num_dofs_per_entity(element)
+    if isinstance(element, SpaceOfReals):
+        return [False for d in num_dofs_per_entity]
+    else:
+        return [d > 0 for d in num_dofs_per_entity]
+
+def _compute_integral_ir(form_data, form_id, parameters, common_cell=None):
     "Compute intermediate represention of form integrals."
 
     irs = []
@@ -201,7 +210,8 @@ def _compute_integral_ir(form_data, form_id, parameters):
                                    metadata,
                                    form_data,
                                    form_id,
-                                   parameters)
+                                   parameters,
+                                   common_cell)
 
         # Append representation
         irs.append(ir)
@@ -214,8 +224,8 @@ def _compute_form_ir(form_data, form_id, element_numbers):
     # Store id
     ir = {"id": form_id}
 
-    print "check 2", element_numbers
-    print [element_numbers[e] for e in form_data.elements]
+    #print "check 2", element_numbers
+    #print [element_numbers[e] for e in form_data.elements]
 
     # Compute common data
     ir["classname"] = "FooForm"
