@@ -11,11 +11,11 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Kristian B. Oelgaard, 2009
 # Modified by Marie E. Rognes, 2010
@@ -27,6 +27,7 @@
 from ufl.classes import Argument
 from ufl.classes import Coefficient
 from ufl.classes import FixedIndex
+from ufl.permutation import build_component_numbering
 
 # FFC modules
 from ffc.log import info, error, ffc_assert
@@ -398,8 +399,14 @@ class TransformedMonomial:
             if c in index_map:
                 index = index_map[c]
             elif isinstance(c, FixedIndex):
+                # Map component using component map from UFL.
+                # KBO: Is this the right place to add, and do we only have
+                # scalar components in the tensor representation at this stage
+                # in the representation?
+                comp_map, comp_num = build_component_numbering(f.function.element().value_shape(), f.function.element().symmetry())
+                comp = comp_map[(int(c),)]
                 index = MonomialIndex(index_type=MonomialIndex.FIXED,
-                                      index_range=[int(c)],
+                                      index_range=[comp],
                                       index_id=None)
             else:
                 index = MonomialIndex(index_range=range(vdim)) # meg: What kind of index should this be?
