@@ -9,17 +9,17 @@
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Garth N. Wells 2006
 # Modified by Kristian B. Oelgaard, 2009.
 #
 # First added:  2004-11-03
-# Last changed: 2010-01-14
+# Last changed: 2011-11-28
 
 # FFC modules.
 from ffc.log import debug
@@ -39,14 +39,16 @@ class ReferenceTensor:
                  monomial,
                  domain_type,
                  facet0, facet1,
-                 quadrature_order):
+                 quadrature_order,
+                 common_cell):
         "Create reference tensor for given monomial."
 
         # Compute reference tensor
         self.A0 = integrate(monomial,
                             domain_type,
                             facet0, facet1,
-                            quadrature_order)
+                            quadrature_order,
+                            common_cell)
 
         # Extract indices
         primary_indices   = monomial.extract_unique_indices(MonomialIndex.PRIMARY)
