@@ -8,7 +8,7 @@ might be (re-)implemented in a future version of FFC
   2. FErari optimizations
 """
 
-# Copyright (C) 2007-2010 Anders Logg
+# Copyright (C) 2007-2011 Anders Logg
 #
 # This file is part of FFC.
 #
@@ -19,16 +19,16 @@ might be (re-)implemented in a future version of FFC
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Kristian B. Oelgaard, 2010.
 #
 # First added:  2007-02-05
-# Last changed: 2010-02-10
+# Last changed: 2011-11-28
 
 # UFL modules
 from ufl.classes import Form, Measure, Integral
@@ -50,7 +50,8 @@ def compute_integral_ir(domain_type,
                         metadata,
                         form_data,
                         form_id,
-                        parameters):
+                        parameters,
+                        common_cell=None):
     "Compute intermediate represention of integral."
 
     info("Computing tensor representation")
@@ -79,7 +80,8 @@ def compute_integral_ir(domain_type,
         ir["AK"] = _compute_terms(monomial_form,
                                   None, None,
                                   domain_type,
-                                  quadrature_degree)
+                                  quadrature_degree,
+                                  common_cell)
 
     elif domain_type == "exterior_facet":
 
@@ -89,7 +91,8 @@ def compute_integral_ir(domain_type,
             terms[i] = _compute_terms(monomial_form,
                                       i, None,
                                       domain_type,
-                                      quadrature_degree)
+                                      quadrature_degree,
+                                      common_cell)
         ir["AK"] = terms
 
     elif domain_type == "interior_facet":
@@ -101,7 +104,8 @@ def compute_integral_ir(domain_type,
                 terms[i][j] = _compute_terms(monomial_form,
                                              i, j,
                                              domain_type,
-                                             quadrature_degree)
+                                             quadrature_degree,
+                                             common_cell)
                 reorder_entries(terms[i][j])
         ir["AK"] = terms
 
@@ -113,7 +117,8 @@ def compute_integral_ir(domain_type,
 def _compute_terms(monomial_form,
                    facet0, facet1,
                    domain_type,
-                   quadrature_degree):
+                   quadrature_degree,
+                   common_cell):
     "Compute list of tensor contraction terms for monomial form."
 
     # Compute terms
@@ -127,7 +132,8 @@ def _compute_terms(monomial_form,
             A0 = ReferenceTensor(monomial,
                                  domain_type,
                                  facet0, facet1,
-                                 quadrature_degree)
+                                 quadrature_degree,
+                                 common_cell)
 
             # Compute geometry tensor
             GK = GeometryTensor(monomial)
