@@ -6,7 +6,7 @@ with stored reference values.
 
 This script can also be used for benchmarking tabulate_tensor for all
 form files found in the 'bench' directory. To run benchmarks, use the
-option -b.
+option --bench.
 """
 
 # Copyright (C) 2010 Anders Logg, Kristian B. Oelgaard and Marie E. Rognes
@@ -20,53 +20,62 @@ option -b.
 #
 # FFC is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public License
-# along with FFC.  If not, see <http://www.gnu.org/licenses/>.
+# along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2010-01-21
-# Last changed: 2010-05-12
+# Last changed: 2011-11-28
 
-# FIXME: Need to add many more test cases. Quite a few DOLFIN
-# FIXME: forms failed after the FFC tests passed. Also need to
-# FIXME: to check with different compiler options, representation
-# FIXME: and DOLFIN wrappers.
+# FIXME: Need to add many more test cases. Quite a few DOLFIN forms
+# failed after the FFC tests passed.
 
-import os, sys, shutil, commands, difflib
-from numpy import array, shape, abs, max
+import os, sys, shutil, difflib
+from numpy import array, shape, abs, max, isnan
 from ffc.log import begin, end, info, info_red, info_green, info_blue
 from ufctest import generate_test_code
+from instant.output import get_status_output
 
 # Parameters
-tolerance = 1e-9
-output_directory = "output"
-demo_directory = "../../../demo"
-bench_directory = "../../../bench"
+output_tolerance = 1.e-6
+demo_directory = "../../../../demo"
+bench_directory = "../../../../bench"
 
 # Global log file
 logfile = None
 
+# Extended quadrature tests (optimisations)
+ext_quad = [\
+"-r quadrature -O -feliminate_zeros",
+"-r quadrature -O -fsimplify_expressions",
+"-r quadrature -O -fprecompute_ip_const",
+"-r quadrature -O -fprecompute_basis_const",
+"-r quadrature -O -fprecompute_ip_const -feliminate_zeros",
+"-r quadrature -O -fprecompute_basis_const -feliminate_zeros",
+]
+
 def run_command(command):
     "Run command and collect errors in log file."
-    (status, output) = commands.getstatusoutput(command)
+    (status, output) = get_status_output(command)
     if status == 0:
         return True
     global logfile
     if logfile is None:
-        logfile = open("../error.log", "w")
+        logfile = open("../../error.log", "w")
     logfile.write(output + "\n")
+    print output
     return False
 
 def log_error(message):
     "Log error message."
     global logfile
     if logfile is None:
-        logfile = open("../error.log", "w")
+        logfile = open("../../error.log", "w")
     logfile.write(message + "\n")
 
-def clean_output():
+def clean_output(output_directory):
     "Clean out old output directory"
     if os.path.isdir(output_directory):
         shutil.rmtree(output_directory)
@@ -112,8 +121,11 @@ def generate_code(args):
     # Iterate over all files
     for f in form_files:
 
+        cmd = ("ffc %s -f precision=8 -fconvert_exceptions_to_warnings %s"
+               % (" ".join(args), f))
+
         # Generate code
-        ok = run_command("ffc %s -v -f precision=8 -fconvert_exceptions_to_warnings %s" % (" ".join(args), f))
+        ok = run_command(cmd)
 
         # Check status
         if ok:
@@ -123,7 +135,7 @@ def generate_code(args):
 
     end()
 
-def validate_code():
+def validate_code(reference_dir):
     "Validate generated code against references."
 
     # Get a list of all files
@@ -139,11 +151,11 @@ def validate_code():
         generated_code = open(f).read()
 
         # Get reference code
-        reference_file = "../references/%s" % f
+        reference_file = os.path.join(reference_dir, f)
         if os.path.isfile(reference_file):
             reference_code = open(reference_file).read()
         else:
-            info_blue("Missing reference for %s" % f)
+            info_blue("Missing reference for %s" % reference_file)
             continue
 
         # Compare with reference
@@ -152,7 +164,8 @@ def validate_code():
         else:
             info_red("%s differs" % f)
             diff = "\n".join([line for line in difflib.unified_diff(reference_code.split("\n"), generated_code.split("\n"))])
-            s = "Code differs for %s, diff follows" % f
+            s = ("Code differs for %s, diff follows"
+                 % os.path.join(*reference_file.split(os.path.sep)[-3:]))
             log_error("\n" + s + "\n" + len(s)*"-")
             log_error(diff)
 
@@ -167,14 +180,50 @@ def build_programs(bench):
 
     begin("Building test programs (%d header files found)" % len(header_files))
 
+    # Get UFC flags
+    ufc_cflags = get_status_output("pkg-config --cflags ufc-1")[1].strip()
+
+    # Get Boost dir (code copied from ufc/src/utils/python/ufc_utils/build.py)
+    # Set a default directory for the boost installation
+    if sys.platform == "darwin":
+        # Use MacPorts as default
+        default = os.path.join(os.path.sep, "opt", "local")
+    else:
+        default = os.path.join(os.path.sep, "usr")
+
+    # If BOOST_DIR is not set use default directory
+    boost_inc_dir = ""
+    boost_lib_dir = ""
+    boost_dir = os.getenv("BOOST_DIR", default)
+    boost_is_found = False
+    for inc_dir in ["", "include"]:
+        if os.path.isfile(os.path.join(boost_dir, inc_dir, "boost", "version.hpp")):
+            boost_inc_dir = os.path.join(boost_dir, inc_dir)
+            break
+    for lib_dir in ["", "lib"]:
+        if os.path.isfile(os.path.join(boost_dir, lib_dir, "libboost_math_tr1.so")) or\
+           os.path.isfile(os.path.join(boost_dir, lib_dir, "libboost_math_tr1.dylib")):
+            boost_lib_dir = os.path.join(boost_dir, lib_dir)
+            break
+    if boost_inc_dir != "" and boost_lib_dir != "":
+        boost_is_found = True
+
+    if not boost_is_found:
+        raise OSError, """The Boost library was not found.
+If Boost is installed in a nonstandard location,
+set the environment variable BOOST_DIR.
+"""
+
+    ufc_cflags += " -I%s -L%s" % (boost_inc_dir, boost_lib_dir)
+
     # Set compiler options
     if bench > 0:
         info("Benchmarking activated")
         # Takes too long to build with -O2
-        #compiler_options = "`pkg-config --cflags ufc-1` -Wall -Werror -O2"
-        compiler_options = "`pkg-config --cflags ufc-1` -Wall -Werror"
+        #compiler_options = "%s -Wall -Werror -O2" % ufc_cflags
+        compiler_options = "%s -Wall -Werror" % ufc_cflags
     else:
-        compiler_options = "`pkg-config --cflags ufc-1` -Wall -Werror -g"
+        compiler_options = "%s -Wall -Werror -g" % ufc_cflags
     info("Compiler options: %s" % compiler_options)
 
     # Iterate over all files
@@ -185,7 +234,7 @@ def build_programs(bench):
 
         # Compile test code
         prefix = f.split(".h")[0]
-        command = "g++ %s -o %s.bin %s.cpp" % (compiler_options, prefix, prefix)
+        command = "g++ %s -o %s.bin %s.cpp -lboost_math_tr1" % (compiler_options, prefix, prefix)
         ok = run_command(command)
 
         # Check status
@@ -210,7 +259,11 @@ def run_programs():
 
         # Compile test code
         prefix = f.split(".bin")[0]
-        ok = run_command("rm -f %s.out; ./%s.bin > %s.out" % (prefix, prefix, prefix))
+        try:
+            os.remove(prefix + ".out")
+        except:
+            pass
+        ok = run_command(".%s%s.bin > %s.out" % (os.path.sep, prefix, prefix))
 
         # Check status
         if ok:
@@ -220,7 +273,7 @@ def run_programs():
 
     end()
 
-def validate_programs():
+def validate_programs(reference_dir):
     "Validate generated programs against references."
 
     # Get a list of all files
@@ -236,18 +289,19 @@ def validate_programs():
         generated_output = open(f).read()
 
         # Get reference output
-        reference_file = "../references/%s" % f
+        reference_file = os.path.join(reference_dir, f)
         if os.path.isfile(reference_file):
             reference_output = open(reference_file).read()
         else:
-            info_blue("Missing reference for %s" % f)
+            info_blue("Missing reference for %s" % reference_file)
             continue
 
         # Compare with reference
         ok = True
         old = [line.split(" = ") for line in reference_output.split("\n") if " = " in line]
         new = dict([line.split(" = ") for line in generated_output.split("\n") if " = " in line])
-        header = "Output differs for %s, diff follows" % f
+        header = ("Output differs for %s, diff follows"
+                  % os.path.join(*reference_file.split(os.path.sep)[-3:]))
         for (key, value) in old:
 
             # Check if value is present
@@ -268,11 +322,11 @@ def validate_programs():
                 ok = False
                 continue
 
-            # Check that values match to within tolerance
+            # Check that values match to within tolerance set by 'output_tolerance'
             diff = max(abs(old_values - new_values))
-            if diff > tolerance:
+            if diff > output_tolerance or isnan(diff):
                 if ok: log_error("\n" + header + "\n" + len(header)*"-")
-                log_error("%s: values differ, error = %g (tolerance = %g)" % (key, diff, tolerance))
+                log_error("%s: values differ, error = %g (tolerance = %g)" % (key, diff, output_tolerance))
                 log_error("  old = " + " ".join("%.16g" % v for v in old_values))
                 log_error("  new = " + " ".join("%.16g" % v for v in new_values))
                 ok = False
@@ -292,37 +346,70 @@ def validate_programs():
 def main(args):
     "Run all regression tests."
 
-    # Clean out old output directory
-    clean_output()
-
-    # Enter output directory
-    os.chdir(output_directory)
-
     # Check command-line arguments
     bench = "--bench" in args
     fast = "--fast" in args
-    args = [arg for arg in args if not arg in ("--bench", "--fast")]
+    ext = "--ext_quad" in args
+    generate_only = "--generate-only" in args
 
-    # Generate test cases
-    generate_test_cases(bench)
+    args = [arg for arg in args
+            if not arg in ("--bench", "--fast", "--ext_quad",
+                           "--generate-only")]
 
-    # Generate code
-    generate_code(args)
+    # Clean out old output directory
+    output_directory = "output"
+    clean_output(output_directory)
+    os.chdir(output_directory)
 
-    # Validate code
-    if not bench:
-        validate_code()
+    # Adjust which test cases (combinations of compile arguments) to
+    # run here
+    test_cases = ["-r auto"]
+    if (not bench and not fast):
+        test_cases += ["-r quadrature", "-r quadrature -O"]
+        if ext:
+            test_cases += ext_quad
 
-    # Build, run and validate programs
-    if fast:
-        info("Skipping program validation")
-    elif bench:
-        build_programs(bench)
-        run_programs()
-    else:
-        build_programs(bench)
-        run_programs()
-        validate_programs()
+    for argument in test_cases:
+
+        begin("Running regression tests with %s" % argument)
+
+        # Clear and enter output sub-directory
+        sub_directory = "_".join(argument.split(" ")).replace("-", "")
+        clean_output(sub_directory)
+        os.chdir(sub_directory)
+
+        # Generate test cases
+        generate_test_cases(bench)
+
+        # Generate code
+        generate_code(args + [argument])
+
+        # Location of reference directories
+        reference_directory =  os.path.abspath("../../references/")
+        code_reference_dir = os.path.join(reference_directory, sub_directory)
+        output_reference_dir = os.path.join(reference_directory, "output")
+
+        # Validate code by comparing to code generated with this set
+        # of compiler parameters
+        if not bench and argument not in ext_quad:
+            validate_code(code_reference_dir)
+
+        # Build and run programs and validate output to common
+        # reference
+        if fast or generate_only:
+            info("Skipping program validation")
+        elif bench:
+            build_programs(bench)
+            run_programs()
+        else:
+            build_programs(bench)
+            run_programs()
+            validate_programs(output_reference_dir)
+
+        # Go back up
+        os.chdir(os.path.pardir)
+
+        end()
 
     # Print results
     if logfile is None:
